<?php
/**
 * @file
 * Croninfinite | croning cron without cron.
 *
 * This is the 'infinite' running croninfinite script
 * that triggers a planned execution of drupals cron.
 *
 * NOTE: This file will not be included by drupal.
 *
 */

/**
 * Ignores the abort of requests and runs till it is
 * killed by configured maximum_execution_time or exit().
 *
 * Without this croninfinite can't respawn independend.
 */
@ignore_user_abort(true);

/**
 * Function to Respawn croninfinite.
 */
function croninfinite_respawn($cron_key, $token, $time_string, $max_execution_time, $temporary_path, $drupal_base_path) {
  // Create the HTTP query to croninfinite.php.
  $request  = "GET " . $_SERVER['PHP_SELF']
    . "?croninfinite_cron_key=" . $cron_key
    . "&croninfinite_token=" . $token
    . "&croninfinite_time_string=" . urlencode($time_string)
    . "&croninfinite_max_execution_time=" . $max_execution_time
    . "&croninfinite_temporary_path=" . $temporary_path
    . "&croninfinite_drupal_base_path=" . $drupal_base_path
    . " HTTP/1.1\r\n";
  $request .= "Host: " . $_SERVER['SERVER_NAME'] . "\r\n";
  $request .= "Connection: Close\r\n\r\n";

  // Open a TCP connection and send the HTTP query.
  $fsock = @fsockopen($_SERVER['SERVER_NAME'], $_SERVER['SERVER_PORT'], $error_no, $error_str, 5);
  if ($fsock) {
    fwrite($fsock, $request);
    fclose($fsock);
  }
}

/**
 * Function to start cron.
 */
function croninfinite_cron_trigger($cron_key, $drupal_base_path) {
  // Execute cron via HTTP request (execution by inclusion could break Croninfinite).
  $request  = "GET " . $drupal_base_path . "cron.php?cron_key=" . $cron_key . " HTTP/1.1\r\n";
  $request .= "Host: " . $_SERVER['SERVER_NAME'] . "\r\n";
  // We don't need to keep alive because cron ignores aborts.
  $request .= "Connection: Close\r\n\r\n";
  // Open a TCP connection and send the HTTP query.
  $fsock = @fsockopen($_SERVER['SERVER_NAME'], $_SERVER['SERVER_PORT'], $error_no, $error_str, 10);
  if ($fsock) {
    fwrite($fsock, $request);
    fclose($fsock);
  }
}

/**
 * Function to check for a planned cron task.
 */
function croninfinite_cron_task($time_string, $lockfile) {
  // Time string syntax: "[*/]#minute(0-59) [*/]#hour(0-23) [#-]#day(1-31) [#-]#month(1-12) [#-]#weekday(0-6)(Sunday=0)".
  // Example for a hourly crontab: "0 * * * *".
  // Example for a crontab on every five minutes: "*/5 * * * *".
  // Example for a crontab on every monday to friday midnight: "0 0 * * 1-5".
  $time_string_array = explode(' ', $time_string);

  $time_string_minute  = &$time_string_array[0];
  $time_string_hour    = &$time_string_array[1];
  $time_string_day     = &$time_string_array[2];
  $time_string_month   = &$time_string_array[3];
  $time_string_weekday = &$time_string_array[4];

  $current_minute  = @idate('i');
  $current_hour    = @idate('H');
  $current_day     = @idate('d');
  $current_month   = @idate('m');
  $current_weekday = @idate('w');

  // Check minute.
  if ($time_string_minute != $current_minute)
    if (@$time_string_minute[1] != '/' || $current_minute % substr($time_string_minute, 2) != 0)
      return FALSE;

  // Check hour.
  if ($time_string_hour != '*' && $time_string_hour != $current_hour)
    if (@$time_string_hour[1] != '/' || $current_hour % substr($time_string_hour, 2) != 0)
      return FALSE;

  // Check day.
  if ($time_string_day != '*' && (string)$time_string_day != (string)$current_day) {
    $time_string_day = explode('-', $time_string_day);
    if (!isset($time_string_day[1]))
      return FALSE;
    if ((int)$time_string_day[0] > (int)$current_day || (int)$time_string_day[1] < (int)$current_day)
      return FALSE;
  }

  // Check month.
  if ($time_string_month != '*' && (string)$time_string_month != (string)$current_month) {
    $time_string_month = explode('-', $time_string_month);
    if (!isset($time_string_month[1]))
      return FALSE;
    if ((int)$time_string_month[0] > (int)$current_month || (int)$time_string_month[1] < (int)$current_month)
      return FALSE;
  }

  // Check weekday.
  if ($time_string_weekday != '*' && (string)$time_string_weekday != (string)$current_weekday) {
    $time_string_weekday = explode('-', $time_string_weekday);
    if (!isset($time_string_weekday[1]))
      return FALSE;
    if ((int)$time_string_weekday[0] > (int)$current_weekday || (int)$time_string_weekday[1] < (int)$current_weekday)
      return FALSE;
  }

  // Check last token change (to be shure not to do one task twice).
  return (int)@file_get_contents($lockfile) != (int)(time()/60);

}

/**
 * Croninfinite main function.
 */
function croninfinite_main() {

  // Grab request information (none of them need to be checked for security reasons).
  $cron_key            = @$_GET['croninfinite_cron_key'];
  $token               = @$_GET['croninfinite_token'];
  $time_string         = @$_GET['croninfinite_time_string'];
  $max_execution_time  = @$_GET['croninfinite_max_execution_time'];
  $temporary_path      = @$_GET['croninfinite_temporary_path'];
  $drupal_base_path    = @$_GET['croninfinite_drupal_base_path'];
  $drupal_root_path    = @$_GET['croninfinite_drupal_root_path'];
  $phase               = @$_GET['croninfinite_phase'];

  // Change current working directory to drupal root.
  @chdir($drupal_root_path);

  // Check for Croninfinite lockfile (also verifies cron_key and token).
  $lockfile = $temporary_path . sha1($cron_key . $token) . '.croninfinite.lock';
  if (!file_exists($lockfile)) {
    exit();
  }

  // Check for planned cron task.
  if (croninfinite_cron_task($time_string, $lockfile)) {

    // Start cron.
    croninfinite_cron_trigger($cron_key, $drupal_base_path);

    // Refresh token change (to be shure not to do one task twice).
    file_put_contents($lockfile, (int)(time()/60));

  }

  // No planned task.
  else {

    // Change logfile every 5 minutes or on start phase, to show that it is (still) running.
    if (@idate('i') % 5 == 0 || $phase == 'start') {
      file_put_contents($lockfile, (int)(time()/60));
    }

    // Sleep almost till the end of execution (before it may get killed).
    if ($phase != 'start') {
      if ($max_execution_time > 6 || $max_execution_time == 0) {
        if ($max_execution_time > 55 || $max_execution_time == 0) {
          $max_execution_time = 55;
        }
        @sleep($max_execution_time - 5);
      }
    }

  }

  // Re-execute the script just before it may gets killed.
  croninfinite_respawn($cron_key, $token, $time_string, $max_execution_time, $temporary_path, $drupal_base_path);

  // Kill it, so there's nothing left to do.
  exit();

}

croninfinite_main();
